
#include <stdio.h>
#include <stdlib.h>

#include "usbapi.h"

#define DBG  0


typedef struct {
	unsigned char Plug5V; // Unplug: 0, Plug: 1 (If have IT6663, this will be always 1)

	unsigned char VideoStable;//Not Stable:0, Stable: 1

	unsigned short Vertical; //Height

	unsigned short Horizontal; // Width

	unsigned char  ColorDepth;// 8, 10, 12

	unsigned char  FrameRate;

	unsigned char  Interlace;// Progressive: 0, Interlace: 1

	unsigned char  HDR; // Off: 0, On: 1

	unsigned char  HDCP; // Off: 0, HDCP 1.x: 1, HDCP 2.x: 2

	unsigned char  SampleRate; // Unknown: 0, 32K: 1, 44.1K: 2, 48K: 3

	unsigned char  InputColorFormat; // RGB444: 0, YUV422: 1, YUV444: 2, YUV420: 3

	unsigned char  VIC;
	unsigned char  RGBQuantization; // Limit: 0, Full: 1
	unsigned char  YUVQuantization; // Limit: 0, Full: 1

	unsigned char  U9RP940A[8];
} YK4K30HDMIINFO;


void enum_all_hid()
{
	struct hid_device_info *devs, *cur_dev;
 	devs = hid_enumerate(0x0, 0x0);
	cur_dev = devs;
	while (cur_dev) {
		printf("Device Found\n  type: %04hx %04hx\n  path: %s\n  serial_number: %ls", cur_dev->vendor_id, cur_dev->product_id, cur_dev->path, cur_dev->serial_number);
		printf("\n");
		printf("  Manufacturer: %ls\n", cur_dev->manufacturer_string);
		printf("  Product:      %ls\n", cur_dev->product_string);
		printf("  Release:      %hx\n", cur_dev->release_number);
		printf("  Interface:    %d\n",  cur_dev->interface_number);
		printf("\n");
		cur_dev = cur_dev->next;
	}
	hid_free_enumeration(devs);
}

int GetHDMISourceInfo(hid_device *handle,int *hdcp_flag)
{
    int res,i;
    char buf[4096];
    memset(buf,0,sizeof(buf));

	// Read a Feature Report from the device
	buf[0] = 0x5;
	res = hid_get_feature_report(handle, buf, 3071);
	if (res < 0) {
		printf("Unable to get a feature report.\n");
		printf("%ls", hid_error(handle));
        return -1;
	}
	else {
#if DBG
		// Print out the returned buffer.
		printf("Feature Report\n   ");
		for (i = 0; i < res; i++)
			printf("%02hhx ", buf[i]);
		printf("\n");
#endif

    char tm = buf[2];
	buf[2] = buf[3];
	buf[3] = tm;
	tm = buf[4];
	buf[4] = buf[5];
	buf[5] = tm;

        YK4K30HDMIINFO *p = (YK4K30HDMIINFO*)buf;
        if(p)
		{
			char buff[256];
			sprintf(buff, "HDMI source info: %hux%hu@%dfps, depth=%d, hdr=%d,hdcp=%d\n", p->Horizontal, p->Vertical,
					p->FrameRate, p->ColorDepth, p->HDR, p->HDCP);
            printf("%s\n",buff);

            if (hdcp_flag)
                *hdcp_flag = p->HDCP;
		}

        return 0;
	}
}

int WriteReg(hid_device *handle,unsigned int addr, unsigned int val)
{
    int res;
	// Send a Feature Report to the device
    char buf[64];
	memset(buf,0,sizeof(buf));
    buf[0] = 0x04;
    buf[1] = 0x0a;
    buf[2] = 0x00;
    *(unsigned int*)&buf[3] = addr;//;
    *(unsigned int*)&buf[7] = val;//;
	res = hid_send_feature_report(handle, buf, 32);
	if (res < 0) {
		printf("Unable to send a feature report.\n");
        return -1;
	}
    else
      return 0;
}

int main()
{
	int res;
	unsigned char buf[4096];
	#define MAX_STR 255
	wchar_t wstr[MAX_STR];
	hid_device *handle;
	int i;
	struct hid_device_info *devs, *cur_dev;
    int flag;


	if (hid_init())
		return -1;

    //enum_all_hid();

	handle = hid_open(0x048d, 0x9325, NULL);
	if (!handle) {
		printf("unable to open device\n");
 		return 1;
	}


	// Read the Product String
	wstr[0] = 0x0000;
	res = hid_get_product_string(handle, wstr, MAX_STR);
	if (res < 0)
		printf("Unable to read product string\n");
	printf("Product String: %ls\n", wstr);

	// Read the Serial Number String
	wstr[0] = 0x0000;
	res = hid_get_serial_number_string(handle, wstr, MAX_STR);
	if (res < 0)
		printf("Unable to read serial number string\n");
	printf("Serial Number String: (%d) %ls", wstr[0], wstr);
	printf("\n");



	// Set the hid_read() function to be non-blocking.
	hid_set_nonblocking(handle, 1);
    flag = 0;
    if (GetHDMISourceInfo(handle,&flag) >=0)
    {
        printf("HDCP FLAG: %d\n",flag);
        printf("Send show video commoand...\n");
        if (WriteReg(handle,0x03FF0000,0xF6000000) >=0)
        {
            printf("Operation OK!!\n");
#if DBG
            GetHDMISourceInfo(handle,&flag);
            printf("HDCP FLAG: %d\n",flag);
#endif // DBG
        }
        else
            printf("Operation failed.\n");
    }


	hid_close(handle);

	/* Free static HIDAPI objects. */
	hid_exit();



}
